package org.rcisoft.wechat;

import me.chanjar.weixin.common.bean.WxJsapiSignature;
import me.chanjar.weixin.common.exception.WxErrorException;
import me.chanjar.weixin.mp.api.WxMpService;
import me.chanjar.weixin.mp.bean.result.WxMpUser;
import me.chanjar.weixin.mp.bean.template.WxMpTemplateData;
import me.chanjar.weixin.mp.bean.template.WxMpTemplateMessage;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.business.mainte.energyplan.entity.BusPushRecord;
import org.rcisoft.business.mainte.energyplan.service.BusPushRecordService;
import org.rcisoft.business.manage.entity.BusProject;
import org.rcisoft.business.manage.service.BusProjectService;
import org.rcisoft.business.system.user.entity.SysUser;
import org.rcisoft.business.system.user.service.UserService;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.rcisoft.core.result.ResultCode;
import org.rcisoft.wechat.service.WxPortalService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 网页授权微信回调接口
 *
 * @author David
 */
@Controller
@RequestMapping("/wechat")
public class WechatRedirectController {

    @Resource
    private WxPortalService service;

    @Autowired
    private WxMpService wxService;

    @Autowired
    private BusProjectService busProjectServiceImpl;

    @Autowired
    private UserService userServiceImpl;

    @Autowired
    private BusPushRecordService busPushRecordServiceImpl;

    @RequestMapping(value = {"/index"})
    public String index(String code) {

        try {
            WxMpUser wxUser = this.service.oauth2UserInfo(code);
            System.out.println("微信返回的code: " + code);
            System.out.println("WxPortalController授权完毕后获取到的OpenId是：" + wxUser.getOpenId());
            return "redirect:http://www.renew-cloud.com/wechat/login.html?openId=" + wxUser.getOpenId();
        } catch (Exception exception) {
            exception.printStackTrace();
            return "redirect:http://www.renew-cloud.com/wechat/error.html";
        }
    }

    @GetMapping(value = {"/getConfig"})
    @ResponseBody
    public Map<String, Object> getConfig(String url) {
        try {
            WxJsapiSignature wxJsapiSignature = service.getJsApiTicket(url);
            Map<String, Object> map = new HashMap<>(4);
            map.put("timestamp", wxJsapiSignature.getTimestamp());
            map.put("nonceStr", wxJsapiSignature.getNonceStr());
            map.put("signature", wxJsapiSignature.getSignature());
            map.put("appId", wxJsapiSignature.getAppId());
            return map;
        } catch (WxErrorException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 推送消息接口
     * @param proId 项目id
     * @param message 推送的消息
     * @return
     */
    @PostMapping(value = {"/push"})
    @ResponseBody
    public Result push(String proId, String message) {
        Date date = new Date();
        String time = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(date);
        BusPushRecord busPushRecord = new BusPushRecord(null, proId, date, message);
        // 发送的结果
        Integer result = 0;
        // 查询项目名称和巡检人ids
        BusProject busProject = busProjectServiceImpl.findByProId(proId);
        // 名称
        String proNm = busProject.getProNm();
        // 巡检人ids
        String inspectIds = busProject.getInspectIds();
        // 根据ids查询巡检人的openId
        List<SysUser> list = userServiceImpl.getUsersByIds(inspectIds);
        if (list != null && list.size() > 0) {
            for (SysUser sysUser : list) {
                String openid = sysUser.getOpenid();
                if (StringUtils.isNotEmpty(openid) && !StringUtils.equals("string", openid)) {
                    result += this.sendMessage(proNm, time, message, openid);
                }
            }
        }
        // 判断发送的信息数量，插入数据
        if (result > 0)
            busPushRecordServiceImpl.add(busPushRecord);
        return Result.builder(new PersistModel(result), MessageConstant.MESSAGE_ALERT_SUCCESS, MessageConstant.MESSAGE_ALERT_ERROR, result);
    }

	/**
	 * 发送模版消息（此方法参数需要修改，需要替换几个参数在此处需要传入几个参数，但是openId是必须的）
	 * @param message
	 * @param openId
	 * @return
	 */
    private Integer sendMessage(String proNm, String time, String message, String openId) {
		WxMpTemplateMessage templateMessage = WxMpTemplateMessage.builder()
				.toUser(openId)
				.templateId("LwdzUKkiSezoITPvgV3bhW7ANnX7f965DBA4FdRkqIw")
//				.url(" ")//详情连接
				.build();

		// 模版中有几个需要替换的参数templateMessage就添加几个WxMpTemplateData
		templateMessage
				.addData(new WxMpTemplateData("first", "调整设备参数", "#FF00FF"))
				.addData(new WxMpTemplateData("keyword1", time, "#FF00FF"))
                .addData(new WxMpTemplateData("keyword2", proNm, "#FF00FF"))
                .addData(new WxMpTemplateData("keyword3", message, "#FF00FF"));
//                .addData(new WxMpTemplateData("remark", message, "#FF00FF"));
        // 消息发送完之后返回的消息id，用于判断是否发送成功
		String msgId;
        try {
        	// 发送消息
			msgId = this.wxService.getTemplateMsgService().sendTemplateMsg(templateMessage);
        } catch (WxErrorException e) {
        	// 此处暂时抛出异常，正式需要注释掉，否则前台不会接到返回值
            e.printStackTrace();
			msgId = "";
        }
        // 判断msgId是否为空，来判断是否发送成功
        if(StringUtils.isNotEmpty(msgId)) {
            return 1;
        } else {
            return 0;
        }

//        return result;
    }
}
